@@ -18,7 +18,10 @@ import json
 import pickle
 import logging
 import traceback
+import threading
+import pandas as pd
 from .mas import Adm
+from fuzzywuzzy import fuzz
 from pydantic import BaseModel
 from typing import List, Optional
 from abc import ABC, abstractmethod
@@ -27,10 +30,14 @@ from pbesa.social.dialog import (
     DialogState, imprimir_grafo, recorrer_interacciones, extraer_diccionario_nodos, 
     ActionNode, DeclarativeNode, GotoNode)
 from .celulas import (celula_casos, celula_consultas, celula_saludos, celula_datos_identificables,
-                      celula_generar_documento, celula_expertos, celula_pertinencia, celula_extraccion,
+                      celula_expertos, celula_pertinencia, celula_extraccion, celula_marco,
                       celula_evaluador, celula_respuesta, celula_conversador, celula_parafraseo, celula_cuestionador,
-                      celula_instruccion, celula_generar_caso, celula_demanda, celula_verificar_calculos)
+                      celula_instruccion, celula_generar_caso, celula_demanda, celula_verificar_calculos,
+                      celula_generar_doc_caso, celula_generar_doc_advice, celula_generar_doc_fact,
+                      celula_generar_doc_goal, celula_generar_doc_test)
 from pbesa.social.prompts import ANALIZER_PROMPT, CLASSIFICATION_PROMPT, DERIVE_PROMPT, RECOVERY_PROMPT, ADAPT_PROMPT, SINTETIZER_PROMPT
+from sklearn.metrics.pairwise import cosine_similarity
+from sklearn.feature_extraction.text import CountVectorizer
 
 # --------------------------------------------------------
 # Define DTOs
@@ -112,6 +119,52 @@ class AgentMetadata():
 # Define common functions
 # --------------------------------------------------------
 
+def calculate_close(text1, text2, ratio) -> bool:
+        """ Check close
+        @param data: Data
+        @return: True if the agent is closed, False otherwise
+        """
+        documents = [text1, text2]
+        count_vectorizer = CountVectorizer()
+        sparse_matrix = count_vectorizer.fit_transform(documents)
+        doc_term_matrix = sparse_matrix.todense()
+        df = pd.DataFrame(
+            doc_term_matrix,
+            columns=count_vectorizer.get_feature_names_out(),
+            index=[text1, text2],
+        )
+        res = cosine_similarity(df, df)
+        similarity = res[0][1]
+        return True if similarity >= ratio else False, similarity
+
+def es_similar(text1, text2):
+    similitud = fuzz.ratio(text1.lower(), text2.lower())
+    return True if similitud >= 79 else False, similitud
+
+def validate_pattern(dto):
+    score_acum = 0
+    patterns = dto['patterns']
+    text_split = dto['text_split']
+    for pattern in patterns:
+        for word in text_split:
+            res, similitud = es_similar(word, pattern)
+            if res:
+                score_acum += similitud
+                break
+    return score_acum / len(patterns) > 80
+
+def evaluate_patterns(patterns_set, text):
+    if text:
+        text_split = text.split(" ")
+        for patterns in patterns_set:
+            dto = {
+                'patterns': patterns,
+                'text_split': text_split
+            }
+            if validate_pattern(dto):
+                return True
+    return False
+
 def define_service_provider(provider, ai_service=None, substitudes = False) -> None:
         # Define provider
         service = None
@@ -298,8 +351,18 @@ class AugmentedGeneration(ABC):
         try:
             if command == "DATA_TYPE":
                 return celula_datos_identificables.derive(self.__ai_service, query)
-            elif command == "GENERATE_DOCUMENT":
-                return celula_generar_documento.derive(self.__ai_service, query["template"], query["text"])
+            
+            elif command == "GENERATE_CASE":
+                return celula_generar_doc_caso.derive(self.__ai_service, query["case"])            
+            elif command == "GENERATE_ADVICE":
+                return celula_generar_doc_advice.derive(self.__ai_service, query["case"])
+            elif command == "GENERATE_FACT":
+                return celula_generar_doc_fact.derive(self.__ai_service, query["case"])
+            elif command == "GENERATE_GOAL":
+                return celula_generar_doc_goal.derive(self.__ai_service, query["case"])
+            elif command == "GENERATE_TEST":
+                return celula_generar_doc_test.derive(self.__ai_service, query["case"])
+            
             elif command == "EXPERTS":
                 retrieval = self.retrieval(query)
                 return celula_expertos.derive(self.__ai_service, query, retrieval, max_tkns)
@@ -699,22 +762,31 @@ class Dialog(ABC):
         self.__vertices = []
 
     def notify(self, session_id, text):
-        try:
-            canales = self.state['canales']
-            canal = canales.get("Webhook")
-            #session_id = self.state['session_id']
-            dto = {
-                "session_id": session_id,
-                "text": text[0:100]
-            }
-            response = canal.post("notify", dto)
-            if response['status']:
-                logging.info(f"Notificación enviada: {text}")
-            else:
-                logging.warning(f"Notificación no enviada: {text}")
-        except Exception as e:
-            logging.error(f"Error al enviar notificación: {text}")
-            logging.error(f"Error: {e}")
+        """Envía una notificación de forma asíncrona usando un hilo que se termina automáticamente"""
+        def _notify_thread():
+            try:
+                canales = self.state['canales']
+                canal = canales.get("Webhook")
+                #session_id = self.state['session_id']
+                dto = {
+                    "session_id": session_id,
+                    "text": text[0:100]
+                }
+                response = canal.post("notify", dto)
+                if response['status']:
+                    logging.info(f"Notificación enviada: {text}")
+                else:
+                    logging.warning(f"Notificación no enviada: {text}")
+            except Exception as e:
+                logging.error(f"Error al enviar notificación: {text}")
+                logging.error(f"Error: {e}")
+            finally:
+                # El hilo se termina automáticamente al finalizar la función
+                logging.debug(f"Hilo de notificación terminado para session_id: {session_id}")
+        
+        # Crear y ejecutar el hilo (se termina automáticamente al finalizar _notify_thread)
+        thread = threading.Thread(target=_notify_thread, daemon=True)
+        thread.start()
     
     def update_conversation(self, session_manager, conversation_id, res):
         conversation = session_manager.get_conversation(conversation_id)
@@ -1495,6 +1567,11 @@ class Dialog(ABC):
                         self.reset()
                         self.notify(session['session_id'], "STOP")
                         return "Web", DialogState.START, res, "Web"
+                
+                elif "Por favor, indíqueme qué" in node.text or "Por favor, proporcione los" in node.text:
+                    res = node.text
+                    logging.info(f"Consulta directa: {res}")
+
                 else:
                     self.__work_memory.append({"role": "system", "content": node.text})
                     logging.info(f"=> !!!!: {query}")
@@ -1614,6 +1691,10 @@ class Dialog(ABC):
             if simulator:
                 simulator.set_service(self.__ai_service)
                 return simulator.derive(query, max_tkns=max_tkns)
+        if command == "EVA_MARCO":
+            return celula_marco.derive(self.__ai_service, query, max_tkns=max_tkns)
+        if command == "EVA_PATTERNS":
+            return validate_pattern(query)
         return None
     
     def parse_conversation(self) -> str:
